#!/usr/bin/perl
# @author poma
# Checks native Unix username/password validity. Workable under any current user.
# usage: auth.pl <username> <password>
# prints to stdout "OK" if success or error string otherwise.

use Authen::PAM;
use POSIX;

# use custom configured PAM service
$service = "ra";
$username = $ARGV[0];

($login,$pass,$uid,$gid) = getpwnam($username) or $quit = true;

if ($quit)
{
    print "User $username does not exist";
    exit;
}

if ($uid == 0)
{
    print "ROOT";
    exit;
}

$password = $ARGV[1];
$tty_name = ttyname(fileno(STDIN));

# function to check password non interactively
sub my_conv_func 
{
    my @res;
    while ( @_ ) 
    {
        my $code = shift;
        my $msg = shift;
        my $ans = "";

        $ans = $username if ($code == PAM_PROMPT_ECHO_ON() );
        $ans = $password if ($code == PAM_PROMPT_ECHO_OFF() );

        push @res, (PAM_SUCCESS(),$ans);
    }
    push @res, PAM_SUCCESS();
    return @res;
}

ref($pamh = new Authen::PAM($service, $username, \&my_conv_func)) ||
         die "Error code $pamh during PAM init!";

$res = $pamh->pam_set_item(PAM_TTY(), $tty_name);
$res = $pamh->pam_authenticate;
if ($res == PAM_SUCCESS())
{
    print "OK";
}
else
{
    print $pamh->pam_strerror($res);
}
